@@ -1,12 +1,12 @@
-import httplib
+import http.client
 import socket
 import sys
 import six
-import urllib
-import urllib2
+import urllib.request, urllib.parse, urllib.error
 
-from codalab.common import http_error_to_exception, precondition, UsageError
-from codalabworker.rest_client import RestClient, RestClientException
+from codalab.common import http_error_to_exception, precondition, ensure_str, UsageError
+from codalab.worker.rest_client import RestClient, RestClientException
+from codalab.worker.download_util import BundleTarget
 
 
 def wrap_exception(message):
@@ -14,9 +14,9 @@ def wrap_exception(message):
         def wrapper(*args, **kwargs):
             try:
                 return f(*args, **kwargs)
-            except urllib2.HTTPError as e:
+            except urllib.error.HTTPError as e:
                 # Translate known errors to the standard CodaLab errors
-                error_body = e.read()
+                error_body = ensure_str(e.read())
                 exc = http_error_to_exception(e.code, error_body)
                 # All standard CodaLab errors are subclasses of UsageError
                 if isinstance(exc, UsageError):
@@ -28,7 +28,7 @@ def wrap_exception(message):
                         JsonApiException(
                             message.format(*args, **kwargs)
                             + ': '
-                            + httplib.responses[e.code]
+                            + http.client.responses[e.code]
                             + ' - '
                             + error_body,
                             400 <= e.code < 500,
@@ -39,11 +39,11 @@ def wrap_exception(message):
                 six.reraise(
                     JsonApiException,
                     JsonApiException(
-                        message.format(*args, **kwargs) + ': ' + e.message, e.client_error
+                        message.format(*args, **kwargs) + ': ' + str(e), e.client_error
                     ),
                     sys.exc_info()[2],
                 )
-            except (urllib2.URLError, httplib.HTTPException, socket.error) as e:
+            except (urllib.error.URLError, http.client.HTTPException, socket.error) as e:
                 six.reraise(
                     JsonApiException,
                     JsonApiException(message.format(*args, **kwargs) + ': ' + str(e), False),
@@ -105,7 +105,7 @@ class JsonApiRelationship(dict):
             dict.__repr__(self),
         )
 
-    def __nonzero__(self):
+    def __bool__(self):
         """
         Implements value of bool(relationship).
         Should be true for non-empty relationships.
@@ -134,7 +134,7 @@ class EmptyJsonApiRelationship(JsonApiRelationship):
         """Empty relationships should be serialized as a null linkage."""
         return {'data': None}
 
-    def __nonzero__(self):
+    def __bool__(self):
         """Empty relationship should be falsey."""
         return False
 
@@ -186,9 +186,9 @@ class JsonApiClient(RestClient):
         if include is not None:
             result.append(('include', ','.join(include)))
 
-        for k, v in params.iteritems() if isinstance(params, dict) else params:
+        for k, v in params.items() if isinstance(params, dict) else params:
             if isinstance(v, list):
-                for item in map(unicode, v):
+                for item in map(str, v):
                     result.append((k, item))
             elif isinstance(v, bool):
                 result.append((k, int(v)))
@@ -286,7 +286,7 @@ class JsonApiClient(RestClient):
                 obj.update(obj_data['attributes'])
             if 'meta' in obj_data:
                 obj['meta'] = obj_data['meta']
-            for key, relationship in obj_data.get('relationships', {}).iteritems():
+            for key, relationship in obj_data.get('relationships', {}).items():
                 linkage = relationship['data']
                 if isinstance(linkage, list):
                     obj[key] = [unpack_linkage(l) for l in linkage]
@@ -371,7 +371,7 @@ class JsonApiClient(RestClient):
             packed_obj = {'type': type_}
             attributes = {}
             relationships = {}
-            for key, value in obj.iteritems():
+            for key, value in obj.items():
                 if isinstance(value, JsonApiRelationship):
                     relationships[key] = value.as_linkage()
                 elif key == 'id':
@@ -443,7 +443,7 @@ class JsonApiClient(RestClient):
         :return: the response
         """
         request_path = '/bundles/%s/netcat/%s/' % (bundle_id, port)
-        return self._make_request('PUT', request_path, data=data)
+        return self._make_request('PUT', request_path, data=data, return_response=True)
 
     @wrap_exception('Unable to create {1}')
     def create(self, resource_type, data, params=None):
@@ -584,26 +584,38 @@ class JsonApiClient(RestClient):
         return response
 
     @wrap_exception('Unable to fetch contents info of bundle {1}')
-    def fetch_contents_info(self, bundle_id, target_path='', depth=0):
-        request_path = '/bundles/%s/contents/info/%s' % (bundle_id, urllib.quote(target_path))
+    def fetch_contents_info(self, target, depth=0):
+        """
+        Calls download_manager.get_target_info server-side and returns the target_info.
+        For details on return value look at worker.download_util.get_target_info
+        :param target: a worker.download_util.BundleTarget
+        """
+        request_path = '/bundles/%s/contents/info/%s' % (
+            target.bundle_uuid,
+            urllib.parse.quote(target.subpath),
+        )
         response = self._make_request('GET', request_path, query_params={'depth': depth})
+        # Deserialize the target. See /rest/bundles/_fetch_contents_info for serialization side
+        response['data']['resolved_target'] = BundleTarget.from_dict(
+            response['data']['resolved_target']
+        )
         return response['data']
 
     @wrap_exception('Unable to fetch contents blob of bundle {1}')
-    def fetch_contents_blob(
-        self, bundle_id, target_path='', range_=None, head=None, tail=None, truncation_text=None
-    ):
+    def fetch_contents_blob(self, target, range_=None, head=None, tail=None, truncation_text=None):
         """
         Returns a file-like object for the target on the given bundle.
 
-        :param bundle_id: id of target bundle
-        :param target_path: path to target in bundle
+        :param target: A worker.download_util.BundleTarget
         :param range_: range of bytes to fetch
         :param head: number of lines to summarize from beginning of file
         :param tail: number of lines to summarize from end of file
         :return: file-like object containing requested data blob
         """
-        request_path = '/bundles/%s/contents/blob/%s' % (bundle_id, urllib.quote(target_path))
+        request_path = '/bundles/%s/contents/blob/%s' % (
+            target.bundle_uuid,
+            urllib.parse.quote(target.subpath),
+        )
         headers = {'Accept-Encoding': 'gzip'}
         if range_ is not None:
             headers['Range'] = 'bytes=%d-%d' % range_
