@@ -35,14 +35,14 @@ from codalab.rest.schemas import (
 )
 from codalab.rest.users import UserSchema
 from codalab.rest.util import get_bundle_infos, get_resource_ids, resolve_owner_in_keywords
-from codalab.server.authenticated_plugin import AuthenticatedPlugin
+from codalab.server.authenticated_plugin import AuthenticatedProtectedPlugin, ProtectedPlugin
 from codalab.worker.bundle_state import State
 from codalab.worker.download_util import BundleTarget
 
 logger = logging.getLogger(__name__)
 
 
-@get('/bundles/<uuid:re:%s>' % spec_util.UUID_STR)
+@get('/bundles/<uuid:re:%s>' % spec_util.UUID_STR, apply=ProtectedPlugin())
 def _fetch_bundle(uuid):
     """
     Fetch bundle by UUID.
@@ -59,7 +59,7 @@ def _fetch_bundle(uuid):
     return document
 
 
-@get('/bundles')
+@get('/bundles', apply=ProtectedPlugin())
 def _fetch_bundles():
     """
     Fetch bundles in the following two ways:
@@ -202,7 +202,7 @@ def build_bundles_document(bundle_uuids):
     return document
 
 
-@post('/bundles', apply=AuthenticatedPlugin())
+@post('/bundles', apply=AuthenticatedProtectedPlugin())
 def _create_bundles():
     """
     Bulk create bundles.
@@ -303,7 +303,7 @@ def _create_bundles():
     return BundleSchema(many=True).dump(bundles).data
 
 
-@patch('/bundles', apply=AuthenticatedPlugin())
+@patch('/bundles', apply=AuthenticatedProtectedPlugin())
 def _update_bundles():
     """
     Bulk update bundles.
@@ -333,7 +333,7 @@ def _update_bundles():
     return BundleSchema(many=True).dump(updated_bundles).data
 
 
-@delete('/bundles', apply=AuthenticatedPlugin())
+@delete('/bundles', apply=AuthenticatedProtectedPlugin())
 def _delete_bundles():
     """
     Delete the bundles specified.
@@ -364,7 +364,7 @@ def _delete_bundles():
     return json_api_meta({}, {'ids': deleted_uuids})
 
 
-@post('/bundle-permissions', apply=AuthenticatedPlugin())
+@post('/bundle-permissions', apply=AuthenticatedProtectedPlugin())
 def _set_bundle_permissions():
     """
     Bulk set bundle permissions.
@@ -429,7 +429,11 @@ def _fetch_bundle_contents_info(uuid, path=''):
     return {'data': info}
 
 
-@put('/bundles/<uuid:re:%s>/netcat/<port:int>/' % spec_util.UUID_STR, name='netcat_bundle')
+@put(
+    '/bundles/<uuid:re:%s>/netcat/<port:int>/' % spec_util.UUID_STR,
+    name='netcat_bundle',
+    apply=ProtectedPlugin(),
+)
 def _netcat_bundle(uuid, port):
     """
     Send a raw bytestring into the specified port of the running bundle with uuid.
@@ -450,22 +454,27 @@ def _netcat_bundle(uuid, port):
 @post(
     '/bundles/<uuid:re:%s>/netcurl/<port:int>/<path:re:.*>' % spec_util.UUID_STR,
     name='netcurl_bundle',
+    apply=ProtectedPlugin(),
 )
 @put(
     '/bundles/<uuid:re:%s>/netcurl/<port:int>/<path:re:.*>' % spec_util.UUID_STR,
     name='netcurl_bundle',
+    apply=ProtectedPlugin(),
 )
 @delete(
     '/bundles/<uuid:re:%s>/netcurl/<port:int>/<path:re:.*>' % spec_util.UUID_STR,
     name='netcurl_bundle',
+    apply=ProtectedPlugin(),
 )
 @get(
     '/bundles/<uuid:re:%s>/netcurl/<port:int>/<path:re:.*>' % spec_util.UUID_STR,
     name='netcurl_bundle',
+    apply=ProtectedPlugin(),
 )
 @patch(
     '/bundles/<uuid:re:%s>/netcurl/<port:int>/<path:re:.*>' % spec_util.UUID_STR,
     name='netcurl_bundle',
+    apply=ProtectedPlugin(),
 )
 def _netcurl_bundle(uuid, port, path=''):
     """
@@ -517,10 +526,15 @@ def _netcurl_bundle(uuid, port, path=''):
         request.path_shift(-4)  # restore the URL
 
 
-@get('/bundles/<uuid:re:%s>/contents/blob/' % spec_util.UUID_STR, name='fetch_bundle_contents_blob')
+@get(
+    '/bundles/<uuid:re:%s>/contents/blob/' % spec_util.UUID_STR,
+    name='fetch_bundle_contents_blob',
+    apply=ProtectedPlugin(),
+)
 @get(
     '/bundles/<uuid:re:%s>/contents/blob/<path:path>' % spec_util.UUID_STR,
     name='fetch_bundle_contents_blob',
+    apply=ProtectedPlugin(),
 )
 def _fetch_bundle_contents_blob(uuid, path=''):
     """
@@ -646,7 +660,7 @@ def _fetch_bundle_contents_blob(uuid, path=''):
 @put(
     '/bundles/<uuid:re:%s>/contents/blob/' % spec_util.UUID_STR,
     name='update_bundle_contents_blob',
-    apply=AuthenticatedPlugin(),
+    apply=AuthenticatedProtectedPlugin(),
 )
 def _update_bundle_contents_blob(uuid):
     """
